/*
 * linux/arch/ppc/kernel/sys_ppc.c
 *
 *  PowerPC version 
 *    Copyright (C) 1995-1996 Gary Thomas (gdt@linuxppc.org)
 *
 *  Derived from "arch/i386/kernel/sys_i386.c"
 *
 * This file contains various random system calls that
 * have a non-standard calling sequence on the Linux/PPC
 * platform.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 *
 */

#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/sem.h>
#include <linux/msg.h>
#include <linux/shm.h>
#include <linux/stat.h>
#include <linux/mman.h>
#include <linux/sys.h>
#include <linux/unistd.h>
#include <asm/ipc.h>
#include <asm/uaccess.h>
#include <asm/ppcops.h>

#ifdef CONFIG_XMON
static inline int sc_enter(int sc) {
	extern int _sc_enter(int, int);
	return _sc_enter(current->pid, sc);
}

static inline void sc_leave(int pos, int rv) {
	extern void _sc_leave(int, int);
	_sc_leave(pos, rv);
}

#else
#define sc_enter(x,y)
#define sc_leave(x,y)
#endif

asmlinkage void syscall(struct pt_regs *regs) {
	extern void syscall_trace(void);
	extern void *sys_call_table[];
	long long (*sc)(unsigned long a1, unsigned long a2, unsigned long a3,
	          unsigned long a4, unsigned long a5, unsigned long a6,
	          struct pt_regs *regs);
	int theCall = regs->gpr[0];
	int rtval;
	long long tval;

#ifdef CONFIG_XMON
	int sc_pos = 0;
        if (theCall != __NR_process_log) sc_pos = sc_enter(theCall);
#endif
	sti();
	regs->ccr &= ~0x10000000;		/* assume no errors ... */
	if (theCall == 0x7777) {
		/*
		 * Restartable system call nonsense.
		 */
		extern int sys_sigreturn(struct pt_regs *);
		rtval = sys_sigreturn(regs);
		if (rtval >= 0) {
			cli();
#ifdef CONFIG_XMON
			if (theCall != __NR_process_log) {
                            sc_leave(sc_pos, regs->gpr[3]);
                        }
#endif
			return;
		}

	restart_check:
		regs->result = rtval;
		if (rtval < 0 && -rtval < _LAST_ERRNO) {
			regs->gpr[3] = (unsigned long) -rtval;
			regs->ccr |= 0x10000000;	/* mark for errno set */
			if (-rtval == ERESTARTNOHAND)
				/* no handler for EINTR restart */
				regs->gpr[3] = EINTR;
		}
		else {
			regs->gpr[3] = (unsigned long) rtval;
                        regs->gpr[4] = (unsigned long) (tval & 0xffffffff);
		}
		if (current->flags & PF_TRACESYS)
			syscall_trace();
		cli();
#ifdef CONFIG_XMON
                if (theCall != __NR_process_log) {
                    sc_leave(sc_pos, regs->gpr[3]);
                }
#endif
		return;
	}
	regs->orig_gpr3 = regs->gpr[3];
	if (current->flags & PF_TRACESYS)
		syscall_trace();
	if (theCall < 0 || theCall > NR_syscalls) {
		/*
		 * Invalid call.
		 */
		regs->gpr[3] = ENOSYS;
		regs->ccr |= 0x10000000;	/* mark for errno set */
		if (current->flags & PF_TRACESYS)
			syscall_trace();
		cli();
#ifdef CONFIG_XMON
                if (theCall != __NR_process_log) {
                    sc_leave(sc_pos, regs->gpr[3]);
                }
#endif
		return;
	}
	sc = (long long (*)(unsigned long, unsigned long, unsigned long,
	              unsigned long, unsigned long, unsigned long,
	              struct pt_regs *)) sys_call_table[theCall];
	if (!sc) {
	    /*
	     * No such call.
	     */
	     regs->gpr[3] = ENOSYS;
	     regs->ccr |= 0x10000000;	/* mark for errno set */
	     if (current->flags & PF_TRACESYS)
		     syscall_trace();
	     cli();
#ifdef CONFIG_XMON
             if (theCall != __NR_process_log) {
                 sc_leave(sc_pos, regs->gpr[3]);
             }
#endif
	     return;
	}
	tval = (*sc)(regs->gpr[3], regs->gpr[4], regs->gpr[5],
		      regs->gpr[6], regs->gpr[7], regs->gpr[8],
		      regs);
	rtval = (int)(tval >> 32);
	goto restart_check;
}

/*
 * sys_pipe() is the normal C calling standard for creating
 * a pipe. It's not the way unix traditionally does this, though.
 */
asmlinkage int sys_pipe(int *fildes)
{
	int error;

	lock_kernel();
	error = verify_area(VERIFY_WRITE, fildes, 8);
	if (error)
		goto out;
	error = do_pipe(fildes);
out:
	unlock_kernel();
	return error;
}

asmlinkage unsigned long sys_mmap(unsigned long addr, size_t len, int prot,
                                  int flags, int fd, off_t offset)
{
	struct file * file = NULL;
	int ret = -EBADF;

	lock_kernel();
	if (!(flags & MAP_ANONYMOUS)) {
		if (fd >= NR_OPEN || !(file = current->files->fd[fd]))
			goto out;
	}
	flags &= ~(MAP_EXECUTABLE | MAP_DENYWRITE);
	ret = do_mmap(file, addr, len, prot, flags, offset);
out:
	unlock_kernel();
	return ret;
}

/*
 * Perform the select(nd, in, out, ex, tv) and mmap() system
 * calls. Linux/i386 didn't use to be able to handle more than
 * 4 system call parameters, so these system calls used a memory
 * block for parameter passing..
 */
asmlinkage int old_mmap(unsigned long *buffer)
{
	int error;
	unsigned long flags;
	long a,b,c,d;
	struct file * file = NULL;

	lock_kernel();
	error = verify_area(VERIFY_READ, buffer, 6*sizeof(long));
	if (error)
		goto out;
	get_user(flags,buffer+3);
	if (!(flags & MAP_ANONYMOUS)) {
		unsigned long fd;
		get_user(fd,buffer+4);
		error = -EBADF;
		if (fd >= NR_OPEN || !(file = current->files->fd[fd]))
			goto out;
	}
	flags &= ~(MAP_EXECUTABLE | MAP_DENYWRITE);
	error = -EFAULT;
	if ( get_user(a,buffer) || get_user(b,buffer+1) ||
	     get_user(c,buffer+2)||get_user(d,buffer+5)	)
		goto out;
	error = do_mmap(file,a,b,c, flags, d);
out:
	unlock_kernel();
	return error;
}

extern asmlinkage int sys_select(int, fd_set *, fd_set *, fd_set *, struct timeval *);

/*
 * This handles syscall 82, which some binaries use with the old
 * calling sequence, and some with the new...
 */
int old_select(int n, fd_set *inp, fd_set *outp, fd_set *exp,
	       struct timeval *tvp)
{
	unsigned long *buffer;

	if ((unsigned)n >= 4096) {
		buffer = (unsigned long *) n;
		if (verify_area(VERIFY_READ, buffer, 5*sizeof(unsigned long))
		    || __get_user(n, buffer)
		    || __get_user(inp, ((fd_set **)(buffer+1)))
		    || __get_user(outp, ((fd_set **)(buffer+2)))
		    || __get_user(exp, ((fd_set **)(buffer+3)))
		    || __get_user(tvp, ((struct timeval **)(buffer+4))))
			return -EFAULT;
	}
	return sys_select(n, inp, outp, exp, tvp);
}

/* The select system call, the easy way... */
int sys_newselect(int n, fd_set *inp, fd_set *outp, fd_set *exp,
		  struct timeval *tvp)
{
	return sys_select(n, inp, outp, exp, tvp);
}

/*
 * sys_ipc() is the de-multiplexer for the SysV IPC calls..
 *
 * This is really horribly ugly.
 */
asmlinkage int 
sys_ipc (uint call, int first, int second, int third, void *ptr, long fifth)
{
	int version, ret;

	lock_kernel();
	version = call >> 16; /* hack for backward compatibility */
	call &= 0xffff;

	ret = -EINVAL;
	switch (call) {
	case SEMOP:
		ret = sys_semop (first, (struct sembuf *)ptr, second);
		break;
	case SEMGET:
		ret = sys_semget (first, second, third);
		break;
	case SEMCTL: {
		union semun fourth;

		if (!ptr)
			break;
		if ((ret = verify_area (VERIFY_READ, ptr, sizeof(long)))
		    || (ret = get_user(fourth.__pad, (void **)ptr)))
			break;
		ret = sys_semctl (first, second, third, fourth);
		break;
		}
	case MSGSND:
		ret = sys_msgsnd (first, (struct msgbuf *) ptr, second, third);
		break;
	case MSGRCV:
		switch (version) {
		case 0: {
			struct ipc_kludge tmp;

			if (!ptr)
				break;
			if ((ret = verify_area (VERIFY_READ, ptr, sizeof(tmp)))
			    || (ret = copy_from_user(&tmp,
						(struct ipc_kludge *) ptr,
						sizeof (tmp))))
				break;
			ret = sys_msgrcv (first, tmp.msgp, second, tmp.msgtyp,
					  third);
			break;
			}
		default:
			ret = sys_msgrcv (first, (struct msgbuf *) ptr,
					  second, fifth, third);
			break;
		}
		break;
	case MSGGET:
		ret = sys_msgget ((key_t) first, second);
		break;
	case MSGCTL:
		ret = sys_msgctl (first, second, (struct msqid_ds *) ptr);
		break;
	case SHMAT:
		switch (version) {
		default: {
			ulong raddr;

			if ((ret = verify_area(VERIFY_WRITE, (ulong*) third,
					       sizeof(ulong))))
				break;
			ret = sys_shmat (first, (char *) ptr, second, &raddr);
			if (ret)
				break;
			ret = put_user (raddr, (ulong *) third);
			break;
			}
		case 1:	/* iBCS2 emulator entry point */
			if (get_fs() != get_ds())
				break;
			ret = sys_shmat (first, (char *) ptr, second,
					 (ulong *) third);
			break;
		}
		break;
	case SHMDT: 
		ret = sys_shmdt ((char *)ptr);
		break;
	case SHMGET:
		ret = sys_shmget (first, second, third);
		break;
	case SHMCTL:
		ret = sys_shmctl (first, second, (struct shmid_ds *) ptr);
		break;
	}

	unlock_kernel();
	return ret;
}
