/*
 * MkLinux pmu stuff.
 *
 * Copyright 1996 1995 by Open Software Foundation, Inc. 1997 1996 1995 1994 1993 1992 1991  
 *
 * Copyright 1996 1995 by Apple Computer, Inc. 1997 1996 1995 1994 1993 1992 1991  
 *
 */

#ifndef _PMU_H_

//-------------------------------
//	VIA-PMU register contents
//-------------------------------

enum {			// port B
    v2PMreq	= 4, 		// Power manager handshake request
    v2PMack	= 3 		// Power manager handshake acknowledge
};

enum {			// IFR/IER
    ifCA2 = 0, 			// CA2 interrupt
    ifCA1 = 1, 			// CA1 interrupt
    ifSR  = 2, 			// SR shift register done
    ifCB2 = 3, 			// CB2 interrupt
    ifCB1 = 4, 			// CB1 interrupt
    ifT2  = 5, 			// T2 timer2 interrupt
    ifT1  = 6, 			// T1 timer1 interrupt
    ifIRQ = 7 			// any interrupt
};


enum
{
    bPMU_ADB_INT	=	4,
    PMU_ADB_INT_MASK	=	(1 << bPMU_ADB_INT)
};


//----------------------------
//	Timers
//----------------------------

enum {
    CONSTANT		= 100,			// nanoseconds per timebase period
    TIMEOUT32MS		= 32000000/CONSTANT,	// 32ms
    TIMEOUT400NS	= 400/CONSTANT		// 400ns
};

typedef struct		// holds the end-time of a timer
{
    u32 upper;
    u32 lower;
} Timer;


/*
 * Results for a_result
 */

#define	ADB_RET_OK			0	/* Successful */
#define	ADB_RET_INUSE			1	/* ADB Device in use */
#define	ADB_RET_NOTPRESENT		2	/* ADB Device not present */
#define ADB_RET_TIMEOUT			3	/* ADB Timeout  */
#define	ADB_RET_UNEXPECTED_RESULT	4	/* Unknown result */
#define	ADB_RET_REQUEST_ERROR		5	/* Packet Request Error */
#define	ADB_RET_BUS_ERROR		6	/* ADB Bus Error */

/*
 * ADB Packet Types
 */

#define	ADB_PACKET_ADB		0
#define	ADB_PACKET_PSEUDO	1
#define	ADB_PACKET_ERROR	2
#define	ADB_PACKET_TIMER	3
#define	ADB_PACKET_POWER	4
#define	ADB_PACKET_MACIIC	5

/*
 * ADB Device Commands 
 */

#define	ADB_ADBCMD_RESET_BUS	0x00
#define	ADB_ADBCMD_FLUSH_ADB	0x01
#define	ADB_ADBCMD_WRITE_ADB	0x08
#define ADB_ADBCMD_READ_ADB	0x0c

/*
 * ADB Pseudo Commands
 */

#define	ADB_PSEUDOCMD_WARM_START		0x00
#define	ADB_PSEUDOCMD_START_STOP_AUTO_POLL	0x01
#define	ADB_PSEUDOCMD_GET_6805_ADDRESS		0x02
#define	ADB_PSEUDOCMD_GET_REAL_TIME		0x03
#define	ADB_PSEUDOCMD_GET_PRAM			0x07
#define	ADB_PSEUDOCMD_SET_6805_ADDRESS		0x08
#define	ADB_PSEUDOCMD_SET_REAL_TIME		0x09
#define	ADB_PSEUDOCMD_POWER_DOWN		0x0a
#define	ADB_PSEUDOCMD_SET_POWER_UPTIME		0x0b
#define	ADB_PSEUDOCMD_SET_PRAM			0x0c
#define	ADB_PSEUDOCMD_MONO_STABLE_RESET		0x0d
#define	ADB_PSEUDOCMD_SEND_DFAC			0x0e
#define	ADB_PSEUDOCMD_BATTERY_SWAP_SENSE	0x10
#define	ADB_PSEUDOCMD_RESTART_SYSTEM		0x11
#define	ADB_PSEUDOCMD_SET_IPL_LEVEL		0x12
#define	ADB_PSEUDOCMD_FILE_SERVER_FLAG		0x13
#define	ADB_PSEUDOCMD_SET_AUTO_RATE		0x14
#define	ADB_PSEUDOCMD_GET_AUTO_RATE		0x16
#define	ADB_PSEUDOCMD_SET_DEVICE_LIST		0x19
#define	ADB_PSEUDOCMD_GET_DEVICE_LIST		0x1a
#define	ADB_PSEUDOCMD_SET_ONE_SECOND_MODE	0x1b
#define	ADB_PSEUDOCMD_SET_POWER_MESSAGES	0x21
#define	ADB_PSEUDOCMD_GET_SET_IIC		0x22
#define	ADB_PSEUDOCMD_ENABLE_DISABLE_WAKEUP	0x23
#define	ADB_PSEUDOCMD_TIMER_TICKLE		0x24
#define	ADB_PSEUDOCMD_COMBINED_FORMAT_IIC	0X25

#endif
